import ws from 'ws';
const WebSocket = global.WebSocket || ws;
function isFillEventUpdate(obj) {
    return obj.event !== undefined;
}
function isHeadUpdate(obj) {
    return obj.head !== undefined;
}
function isStatusMessage(obj) {
    return obj.success !== undefined;
}
export class FillsFeed {
    _url;
    _socket;
    _subscriptions;
    _connected;
    _reconnectionIntervalMs;
    _reconnectionAttempts;
    _reconnectionMaxAttempts;
    _onConnect = null;
    _onDisconnect = null;
    _onFill = null;
    _onHead = null;
    _onStatus = null;
    constructor(url, options) {
        this._url = url;
        this._subscriptions = options?.subscriptions;
        this._reconnectionIntervalMs = options?.reconnectIntervalMs ?? 5000;
        this._reconnectionAttempts = 0;
        this._reconnectionMaxAttempts = options?.reconnectionMaxAttempts ?? -1;
        this._connect();
    }
    _reconnectionAttemptsExhausted() {
        return (this._reconnectionMaxAttempts != -1 &&
            this._reconnectionAttempts >= this._reconnectionMaxAttempts);
    }
    _connect() {
        this._socket = new WebSocket(this._url);
        this._socket.addEventListener('error', (err) => {
            console.warn(`[FillsFeed] connection error: ${err.message}`);
            if (this._reconnectionAttemptsExhausted()) {
                console.error('[FillsFeed] fatal connection error');
                throw err.error;
            }
        });
        this._socket.addEventListener('open', () => {
            if (this._subscriptions !== undefined) {
                this._connected = true;
                this._reconnectionAttempts = 0;
                this.subscribe(this._subscriptions);
            }
            if (this._onConnect)
                this._onConnect();
        });
        this._socket.addEventListener('close', () => {
            this._connected = false;
            setTimeout(() => {
                if (!this._reconnectionAttemptsExhausted()) {
                    this._reconnectionAttempts++;
                    this._connect();
                }
            }, this._reconnectionIntervalMs);
            if (this._onDisconnect)
                this._onDisconnect();
        });
        this._socket.addEventListener('message', (msg) => {
            try {
                const data = JSON.parse(msg.data);
                if (isFillEventUpdate(data) && this._onFill) {
                    this._onFill(data);
                }
                else if (isHeadUpdate(data) && this._onHead) {
                    this._onHead(data);
                }
                else if (isStatusMessage(data) && this._onStatus) {
                    this._onStatus(data);
                }
            }
            catch (err) {
                console.warn('[FillsFeed] error deserializing message', err);
            }
        });
    }
    subscribe(subscriptions) {
        if (this._connected) {
            this._socket.send(JSON.stringify({
                command: 'subscribe',
                ...subscriptions,
            }));
        }
        else {
            console.warn('[FillsFeed] attempt to subscribe when not connected');
        }
    }
    unsubscribe(marketId) {
        if (this._connected) {
            this._socket.send(JSON.stringify({
                command: 'unsubscribe',
                marketId,
            }));
        }
        else {
            console.warn('[FillsFeed] attempt to unsubscribe when not connected');
        }
    }
    connected() {
        return this._connected;
    }
    onConnect(callback) {
        this._onConnect = callback;
    }
    onDisconnect(callback) {
        this._onDisconnect = callback;
    }
    onFill(callback) {
        this._onFill = callback;
    }
    onHead(callback) {
        this._onHead = callback;
    }
    onStatus(callback) {
        this._onStatus = callback;
    }
}
